! halo model nonlinear fitting formula as described in 
! Appendix C of Smith et al. (2002)
subroutine halofit(redshift,rk,rn,rncur,rknl,plin,pnl)
  USE cosmo
  implicit none
  double precision, intent(IN) :: redshift,rk,rn,rncur,rknl,plin
  double precision, intent(OUT):: pnl
  double precision :: gam,a,amod,b,c,xmu,xnu,alpha,beta,f1,f2,f3,f4
  double precision :: pq,ph
  double precision :: om_m,om_v,aa,omega_t
  double precision :: y
  double precision :: f1a,f2a,f3a,f4a,f1b,f2b,f3b,f4b,frac
  aa=1d0/(1d0+redshift)
  omega_t=1.0+(om0+ode0-1.0)/(1-om0-ode0+ode0*aa*aa+om0/aa)
  om_m=omega_t*om0/(om0+ode0*aa*aa*aa)
  om_v=omega_t*ode0/(ode0+om0/aa/aa/aa)

  gam=0.86485+0.2989*rn+0.1631*rncur
  a=1.4861+1.83693*rn+1.67618*rn*rn+0.7940*rn*rn*rn+0.1670756*rn*rn*rn*rn-0.620695*rncur
  a=10**a      
  b=10**(0.9463+0.9466*rn+0.3084*rn*rn-0.940*rncur)
  c=10**(-0.2807+0.6669*rn+0.3214*rn*rn-0.0793*rncur)
  xmu=10**(-3.54419+0.19086*rn)
  xnu=10**(0.95897+1.2857*rn)
  alpha=1.38848+0.3701*rn-0.1452*rn*rn
  beta=0.8291+0.9854*rn+0.3400*rn**2

  if(abs(1-om_m).gt.0.01) then ! omega evolution 
     f1a=om_m**(-0.0732)
     f2a=om_m**(-0.1423)
     f3a=om_m**(0.0725)
     f1b=om_m**(-0.0307)
     f2b=om_m**(-0.0585)
     f3b=om_m**(0.0743)       
     frac=om_v/(1.-om_m) 
     f1=frac*f1b + (1-frac)*f1a
     f2=frac*f2b + (1-frac)*f2a
     f3=frac*f3b + (1-frac)*f3a
  else         
     f1=1.0
     f2=1.
     f3=1.
  endif

  y=(rk/rknl)
  
  ph=a*y**(f1*3)/(1+b*y**(f2)+(f3*c*y)**(3-gam))
  ph=ph/(1+xmu*y**(-1)+xnu*y**(-2))
  pq=plin*(1+plin)**beta/(1+plin*alpha)*exp(-y/4.0-y**2/8.0)
  
  pnl=pq+ph
  
  return
end subroutine halofit
