!-----------------------------------------------------------------------------
!
! Compute sigma^2(R) from the linear matter power spectrum 
! WITH A GAUSSIAN WINDOW, and fit it by Chebyshev polynomials 
! (single precision)
!
! << sample code >>
! 
! USE linearpk
! USE sigmagaus
! REAL :: chebev
! REAL :: lnsigma2,lnRh,Rh,dlnsigma2dlnRh
! character(len=128) :: filename
! integer :: n
! filename='wmap5baosn_max_likelihood_matterpower.dat'
! n=896 ! # of lines in the file
! CALL get_linearpk(filename,n)
! CALL compute_sigma2gaus
! Rh=8. ! h^-1 Mpc
! lnRh = alog(Rh)
! if(lnRh>lnR2.or.lnRh<lnR1)then
!    print*,'lnR=',lnRh,' is out of range. It must be between',lnR1,' and',lnR2
!    stop
! else
!    lnsigma2 = CHEBEV(lnR1,lnR2,cg,ndim,lnRh)
!    print*,'sigma(R) at R=',Rh,' h^-1 Mpc is',exp(0.5*lnsigma2)
!    dlnsigma2dlnRh = CHEBEV(lnR1,lnR2,cgder,ndim,lnRh)
!    print*,'dln(sigma)/dlnR at R=',Rh,' h^-1 Mpc is',0.5*dlnsigma2dlnRh
! endif
! end
!
! January 1, 2012
! E. Komatsu
!
!-----------------------------------------------------------------------------
MODULE sigmagaus
  INTEGER :: ndimg=30
  REAL    :: cg(30),cgder(30),cgderder(30)
  REAL    :: lnR1g=-5.684 ! 0.0034Mpc/h, 1.8e4  solar mass
  REAL    :: lnR2g=4.     ! 54.9Mpc/h, 7.5e16 solar mass
contains
  SUBROUTINE compute_sigma2gaus
    REAL :: lnsigma2gaus
    EXTERNAL lnsigma2gaus
    CALL chebft(lnR1g,lnR2g,cg,ndimg,lnsigma2gaus)
    CALL chder(lnR1g,lnR2g,cg,cgder,ndimg)
    CALL chder(lnR1g,lnR2g,cgder,cgderder,ndimg)
    return
  END SUBROUTINE compute_sigma2gaus
END MODULE sigmagaus
!--------------------------------------------
REAL FUNCTION lnsigma2gaus(lnR)
  IMPLICIT none
  REAL :: lnR
  DOUBLE PRECISION :: sigma2,r
  DOUBLE PRECISION :: ds2dk
  EXTERNAL ds2dkgaus
  r = exp(lnR)
  CALL qromb(ds2dkgaus,dlog(1d-4),dlog(2d1/r),sigma2,r)
  lnsigma2gaus = alog(REAL(sigma2))
  return
END FUNCTION lnsigma2gaus
!--------------------------------------------
DOUBLE PRECISION FUNCTION ds2dkgaus(lnk,r)
  IMPLICIT none
  DOUBLE PRECISION :: lnk,k,r,x
  DOUBLE PRECISION :: linear_pk,win
  DOUBLE PRECISION :: pi= 3.14159265d0
  k = dexp(lnk)
  x = k*r
!  win= (3d0/x)*(sin(x)/x**2d0-cos(x)/x)
  win=dexp(-0.5d0*x**2d0)
  ds2dkgaus = k**3d0*(linear_pk(k)*win**2d0)/(2d0*pi**2d0)
  return
END FUNCTION ds2dkgaus
